#!/usr/bin/env python
from flask import Flask, render_template, session, request, Response
from flask_socketio import SocketIO, emit, disconnect
import time
import gevent
from threading import Thread
from camera import VideoCamera
from Motor_Control import motorcontrol

# Set this variable to "threading", "eventlet" or "gevent" to test the
# different async modes, or leave it set to None for the application to choose
# the best option based on installed packages.
async_mode = None

app = Flask(__name__)
app.config['SECRET_KEY'] = 'secret!'
socketio = SocketIO(app, async_mode=async_mode)
thread = 5
trackfreq=10000
mc=motorcontrol()
def background_thread():
    """Example of how to send server generated events to clients."""
    count = 0
   
#        while True:
#        socketio.sleep(10)
#        count += 1
#        socketio.emit('my_response',
#                      {'data': 'Server generated Event', 'count': count},
#                      namespace='/test')

@app.route('/')
def index():
    return render_template('index.html', async_mode=socketio.async_mode)

def gen(camera):
    # the timestart variable is an attempt to limit calls to get a camera frame 1/.0333 (30) times per second to match the camera limit
    timestart =time.clock()+.033333
    while True:
        #if time.clock() > timestart:
        #   timestart=time.clock()+.033333
            frame = camera.get_frame()
            yield (b'--frame\r\n'b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')
            gevent.sleep(.033333)

@app.route('/video_feed')
def video_feed():
    """Video streaming route. Put this in the src attribute of an img tag."""
    return Response(gen(VideoCamera()),
                    mimetype='multipart/x-mixed-replace; boundary=frame')

@socketio.on('my_event', namespace='/test')
def test_message(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']})

@socketio.on('forward_event', namespace='/test')
def forward(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.forward(1,1,trackfreq)
    gevent.sleep(float(rate)/20)
    mc.forward(0,0,trackfreq)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)

@socketio.on('reverse_event', namespace='/test')
def reverse(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.forward(0,1,trackfreq)
    gevent.sleep(float(rate)/20)
    mc.forward(0,0,trackfreq)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)

@socketio.on('left_event', namespace='/test')
def left(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.left(1,1,trackfreq)
    gevent.sleep(float(rate)/20)
    mc.left(0,0,trackfreq)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)
         
@socketio.on('right_event', namespace='/test')
def right(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.right(1,1,trackfreq)
    gevent.sleep(float(rate)/20)
    mc.right(0,0,trackfreq)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)

@socketio.on('panleft_event', namespace='/test')
def panleft(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.panmotor(1,1)
    gevent.sleep(float(rate)/20)
    mc.panmotor(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)
         
@socketio.on('panright_event', namespace='/test')
def panright(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.panmotor(0,1)
    gevent.sleep(float(rate)/20)
    mc.panmotor(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)
         
@socketio.on('tiltup_event', namespace='/test')
def tiltup(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.tiltmotor(0,1)
    gevent.sleep(float(rate)/20)
    mc.tiltmotor(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)

@socketio.on('tiltdown_event', namespace='/test')
def tiltdown(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.tiltmotor(1,1)
    gevent.sleep(float(rate)/20)
    mc.tiltmotor(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)

@socketio.on('arm1forward_event', namespace='/test')
def arm1forward(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.arm1(1,1)
    gevent.sleep(float(rate)/20)
    mc.arm1(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)
         
@socketio.on('arm1reverse_event', namespace='/test')
def arm1reverse(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.arm1(0,1)
    gevent.sleep(float(rate)/20)
    mc.arm1(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)

@socketio.on('arm2forward_event', namespace='/test')
def arm2forward(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.arm2(0,1)
    gevent.sleep(float(rate)/20)
    mc.arm2(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)
         
@socketio.on('arm2reverse_event', namespace='/test')
def arm2reverse(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.arm2(1,1)
    gevent.sleep(float(rate)/20)
    mc.arm2(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)
         

@socketio.on('arm3forward_event', namespace='/test')
def arm3forward(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.arm3(0,1)
    gevent.sleep(float(rate)/20)
    mc.arm3(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)
         
@socketio.on('arm3reverse_event', namespace='/test')
def arm3reverse(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    rate=message['data']
    print rate
    mc.arm3(1,1)
    gevent.sleep(float(rate)/20)
    mc.arm3(0,0)
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)         


@socketio.on('trackfreq_event', namespace='/test')
def trackfrequency(message):
    session['receive_count'] = session.get('receive_count', 0) + 1
    global trackfreq
    trackfreq=message['data']
    print trackfreq
    emit('my_response',
         {'data': message['data'], 'count': session['receive_count']},
         broadcast=True)   
         
@socketio.on('cutpower_request', namespace='/test')
def cutpower_request():
    session['receive_count'] = session.get('receive_count', 0) + 1
    print "cut"
    mc.cutpower()
    emit('my_response',
         {'data': 'power cut', 'count': session['receive_count']})     
         
@socketio.on('my_ping', namespace='/test')
def ping_pong():
    emit('my_pong')


@socketio.on('connect', namespace='/test')
def test_connect():
    global thread
    if thread is None:
        thread = socketio.start_background_task(target=background_thread)
    emit('my_response', {'data': 'Connected', 'count': 1})



if __name__ == '__main__':
    socketio.run(app, host='0.0.0.0',debug=False)
    
